@@ -1,5 +1,20 @@
+__license__ = """
+NML is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
+
+NML is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License along
+with NML; if not, write to the Free Software Foundation, Inc.,
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
+
 from nml import generic
-from nml.expression import ConstantNumeric, Array, StringLiteral, Identifier
+from nml.expression import ConstantNumeric, ConstantFloat, Array, StringLiteral, Identifier
 
 tilelayout_names = {}
 
@@ -26,6 +41,8 @@ class Action0Property(object):
                 biggest = 1 << (8 * size)
                 if val.value >= biggest:
                     raise generic.ScriptError("Action 0 property too large", val.pos)
+                elif val.value < 0 and val.value + (biggest / 2) < 0:
+                    raise generic.ScriptError("Action 0 property too small", val.pos)
 
     def write(self, file):
         file.print_bytex(self.num)
@@ -196,6 +213,9 @@ general_veh_props = {
     'name'               : {'num': -1, 'string': None},
 }
 
+def ottd_display_speed(value, divisor, unit):
+    return (value.value / divisor * 10 / 16 * unit.ottd_mul) >> unit.ottd_shift
+
 #
 # Feature 0x00 (Trains)
 #
@@ -203,7 +223,7 @@ general_veh_props = {
 properties[0x00] = {
     'track_type'                   : {'size': 1, 'num': 0x05},
     'ai_special_flag'              : {'size': 1, 'num': 0x08},
-    'speed'                        : {'size': 2, 'num': 0x09, 'unit_type': 'speed', 'unit_conversion': 3.5790976},
+    'speed'                        : {'size': 2, 'num': 0x09, 'unit_type': 'speed', 'unit_conversion': 3.5790976, 'adjust_value': lambda val, unit: ottd_display_speed(val, 1, unit)},
     'power'                        : {'size': 2, 'num': 0x0B, 'unit_type': 'power'},
     'running_cost_factor'          : {'size': 1, 'num': 0x0D},
     'running_cost_base'            : {'size': 4, 'num': 0x0E},
@@ -229,6 +249,7 @@ properties[0x00] = {
     'refittable_cargo_classes'     : {'size': 2, 'num': 0x28, 'append_function': append_cargo_type(0x00)},
     'non_refittable_cargo_classes' : {'size': 2, 'num': 0x29, 'append_function': append_cargo_type(0x00)},
     'introduction_date'            : {'size': 4, 'num': 0x2A},
+    'cargo_age_period'             : {'size': 2, 'num': 0x2B},
 }
 properties[0x00].update(general_veh_props)
 
@@ -246,7 +267,7 @@ def roadveh_speed_prop(value):
     return props
 
 properties[0x01] = {
-    'speed'                        : {'custom_function' : roadveh_speed_prop, 'unit_type': 'speed', 'unit_conversion': 7.1581952},
+    'speed'                        : {'custom_function' : roadveh_speed_prop, 'unit_type': 'speed', 'unit_conversion': 7.1581952, 'adjust_value': lambda val, unit: ottd_display_speed(val, 2, unit)},
     'running_cost_factor'          : {'size': 1, 'num': 0x09},
     'running_cost_base'            : {'size': 4, 'num': 0x0A},
     'sprite_id'                    : {'size': 1, 'num': 0x0E},
@@ -265,6 +286,8 @@ properties[0x01] = {
     'refittable_cargo_classes'     : {'size': 2, 'num': 0x1D, 'append_function': append_cargo_type(0x01)},
     'non_refittable_cargo_classes' : {'size': 2, 'num': 0x1E, 'append_function': append_cargo_type(0x01)},
     'introduction_date'            : {'size': 4, 'num': 0x1F},
+    'visual_effect'                : {'size': 1, 'num': 0x21},
+    'cargo_age_period'             : {'size': 2, 'num': 0x22},
 }
 properties[0x01].update(general_veh_props)
 
@@ -272,24 +295,35 @@ properties[0x01].update(general_veh_props)
 # Feature 0x02 (Ships)
 #
 
+def speed_fraction_prop(value, propnr):
+    # Unit is already converted to 0 .. 255 range when we get here
+    value = value.reduce_constant()
+    if not (0 <= value.value <= 255):
+        # Do not use check_range to provide better error message
+        raise generic.ScriptError("speed fraction must be in range 0 .. 1", value.pos)
+    value = ConstantNumeric(255 - value.value, value.pos)
+    return [Action0Property(propnr, value, 1)]
+
 properties[0x02] = {
     'sprite_id'                    : {'size': 1, 'num': 0x08},
     'is_refittable'                : {'size': 1, 'num': 0x09},
     'cost_factor'                  : {'size': 1, 'num': 0x0A},
-    'speed'                        : {'size': 1, 'num': 0x0B, 'unit_type': 'speed', 'unit_conversion': 7.1581952},
+    'speed'                        : {'size': 1, 'num': 0x0B, 'unit_type': 'speed', 'unit_conversion': 7.1581952, 'adjust_value': lambda val, unit: ottd_display_speed(val, 2, unit)},
     'cargo_capacity'               : {'size': 2, 'num': 0x0D},
     'running_cost_factor'          : {'size': 1, 'num': 0x0F},
     'sound_effect'                 : {'size': 1, 'num': 0x10},
     'refittable_cargo_types'       : {'size': 4, 'num': 0x11, 'append_function': append_cargo_type(0x02)},
     'callback_flags'               : {'size': 1, 'num': 0x12},
-    'refit_cost'                   : {'size': 1, 'num': 0x15},
-    'ocean_speed_fraction'         : {'size': 1, 'num': 0x14},
-    'canal_speed_fraction'         : {'size': 1, 'num': 0x15},
+    'refit_cost'                   : {'size': 1, 'num': 0x13},
+    'ocean_speed_fraction'         : {'size': 1, 'num': 0x14, 'unit_conversion': 255, 'custom_function': lambda val: speed_fraction_prop(val, 0x14)},
+    'canal_speed_fraction'         : {'size': 1, 'num': 0x15, 'unit_conversion': 255, 'custom_function': lambda val: speed_fraction_prop(val, 0x15)},
     'retire_early'                 : {'size': 1, 'num': 0x16},
     'misc_flags'                   : {'size': 1, 'num': 0x17},
     'refittable_cargo_classes'     : {'size': 2, 'num': 0x18, 'append_function': append_cargo_type(0x02)},
     'non_refittable_cargo_classes' : {'size': 2, 'num': 0x19, 'append_function': append_cargo_type(0x02)},
     'introduction_date'            : {'size': 4, 'num': 0x1A},
+    'visual_effect'                : {'size': 1, 'num': 0x1C},
+    'cargo_age_period'             : {'size': 2, 'num': 0x1D},
 }
 properties[0x02].update(general_veh_props)
 
@@ -302,7 +336,7 @@ properties[0x03] = {
     'is_helicopter'                : {'size': 1, 'num': 0x09},
     'is_large'                     : {'size': 1, 'num': 0x0A},
     'cost_factor'                  : {'size': 1, 'num': 0x0B},
-    'speed'                        : {'size': 1, 'num': 0x0C, 'unit_type': 'speed', 'unit_conversion': 0.279617},
+    'speed'                        : {'size': 1, 'num': 0x0C, 'unit_type': 'speed', 'unit_conversion': 0.279617, 'adjust_value': lambda val, unit: ottd_display_speed(val, 1, unit)},
     'acceleration'                 : {'size': 1, 'num': 0x0D},
     'running_cost_factor'          : {'size': 1, 'num': 0x0E},
     'passenger_capacity'           : {'size': 2, 'num': 0x0F},
@@ -316,11 +350,18 @@ properties[0x03] = {
     'refittable_cargo_classes'     : {'size': 2, 'num': 0x18},
     'non_refittable_cargo_classes' : {'size': 2, 'num': 0x19},
     'introduction_date'            : {'size': 4, 'num': 0x1A},
+    'cargo_age_period'             : {'size': 2, 'num': 0x1C},
+    'range'                        : {'size': 2, 'num': 0x1F},
 }
 properties[0x03].update(general_veh_props)
 
 # TODO: Feature 0x04 .. 0x06 (Stations, Canals, Bridges)
 
+properties[0x05] = {
+    'callback_flags' : {'size': 1, 'num': 0x08},
+    'graphic_flags'  : {'size': 1, 'num': 0x09},
+}
+
 #
 # Feature 0x07 (Houses)
 #
@@ -505,8 +546,14 @@ def industry_conflicting_types(value):
 def industry_input_multiplier(value, prop_num):
     if not isinstance(value, Array) or len(value.values) > 2:
         raise generic.ScriptError("Input multiplier must be an array of up to two values", value.pos)
-    mul1 = value.values[0].reduce_constant().value * 256 if len(value.values) > 0 else 0
-    mul2 = value.values[1].reduce_constant().value * 256 if len(value.values) > 1 else 0
+    val1 = value.values[0].reduce() if len(value.values) > 0 else ConstantNumeric(0)
+    val2 = value.values[1].reduce() if len(value.values) > 1 else ConstantNumeric(0)
+    if not isinstance(val1, (ConstantNumeric, ConstantFloat)) or not isinstance(val2, (ConstantNumeric, ConstantFloat)):
+        raise generic.ScriptError("Expected a compile-time constant", value.pos)
+    generic.check_range(val1.value, 0, 256, "input_multiplier", val1.pos)
+    generic.check_range(val2.value, 0, 256, "input_multiplier", val2.pos)
+    mul1 = int(val1.value * 256)
+    mul2 = int(val2.value * 256)
     return [Action0Property(prop_num, ConstantNumeric(mul1 | (mul2 << 16)), 4)]
 
 properties[0x0A] = {
@@ -658,6 +705,7 @@ properties[0x0F] = {
     'remove_cost_multiplier' : {'size': 1, 'num': 0x14},
     'callback_flags'         : {'size': 2, 'num': 0x15},
     'height'                 : {'size': 1, 'num': 0x16},
+    'num_views'              : {'size': 1, 'num': 0x17},
 }
 
 #
